@@ -1,5 +1,4 @@
 from functools import partial
-import re
 
 from pandas import DataFrame, Series
 import numpy as np
@@ -23,6 +22,7 @@ period_to_int = partial(to_int, 'period', 1)
 times_to_int = partial(to_int, 'times', 0)
 repeat_to_int = partial(to_int, 'repeat', 0)
 
+
 def create_enum(choices: list, name: str, value: str):
     if value in choices:
         return value
@@ -33,58 +33,47 @@ def create_enum(choices: list, name: str, value: str):
         f'{name} should be either {choices_str}, but got `{value}`'
     )
 
+
 style_enums = partial(create_enum, [
     'bullish',
     'bearish'
 ], 'style')
 
+column_enums = partial(create_enum, [
+    'open',
+    'high',
+    'low',
+    'close'
+], 'column')
+
 
 def to_direction(value: str) -> int:
-    if direction == '1':
+    if value == '1':
         return 1
 
-    if direction == '-1':
+    if value == '-1':
         return - 1
 
     raise ValueError(f'direction must be `1` or `-1`, but got `{value}`')
 
 
-COLUMNS = [
-    'open',
-    'high',
-    'low',
-    'close'
-]
+# A simple cache
+class DirectiveCache:
+    def __init__(self):
+        self._store = {}
 
+    def set(self, key: str, value):
+        self._store[key] = value
 
-def is_ohlc_column(column: str):
-    return column in COLUMNS
-
-
-def is_valid_stat_column(column: str):
-    if is_ohlc_column(column):
-        return column
-
-    raise ValueError(
-        f'"{column}" is not a valid column for statistics'
-    )
-
-
-def memoize(f):
-    memo = {}
-
-    def helper(x):
-        if x not in memo:
-            memo[x] = f(x)
-        return memo[x]
-    return helper
+    def get(self, key: str, default=None):
+        return self._store.get(key, default)
 
 
 def set_stock_metas(
     target,
     aliases_map={},
     columns_info_map={},
-    directives_cache={}
+    directives_cache=DirectiveCache()
 ):
     # Use `object.__setattr__` to avoid pandas UserWarning:
     # > Pandas doesn't allow columns to be created via a new attribute name
@@ -116,11 +105,6 @@ def copy_stock_metas(source, target):
     )
 
 
-def raise_if(strict: bool, err):
-    if strict:
-        raise err
-
-
 def ensure_return_type(cls, method):
     def helper(self, *args, **kwargs):
         ret = getattr(super(cls, self), method)(*args, **kwargs)
@@ -155,61 +139,12 @@ def is_not_nan(subject):
     return ~ np.isnan(subject)
 
 
-# left parentheses
-PARAN_L = '('
-PARAN_R = ')'
 ARGS_SEPARATOR = ','
 
-def balance(l: list, strict: bool):
-    balanced = []
-
-    pending = None
-
-    # ['1', '( foo:1', '3', '2  )']
-    for item in l:
-        if pending is None:
-            if item.startswith(PARAN_L):
-                # '( foo:1' -> 'foo:1'
-                pending = item[1:].strip()
-            else:
-                balanced.append(item)
-        else:
-            if item.endswith(PARAN_R):
-                balanced.append(
-                    # foo:1,3 ,                2
-                    pending + ARGS_SEPARATOR + item[:- 1].strip()
-                )
-                pending = None
-            else:
-                # foo:1    ,                3
-                pending += ARGS_SEPARATOR + item
-
-    if pending is not None:
-        return raise_if(
-            strict,
-            ValueError(f'unbalanced argument paranthesis "({pending}"')
-        )
-
-    return balanced
-
-
-def split_and_balance(args_str: str, strict: bool) -> list:
-    splitted = [
-        a.strip() for a in args_str.split(ARGS_SEPARATOR)
-    ]
-
-    return balance(splitted, strict) if PARAN_L in args_str else splitted
-
-
-def quote_arg(arg):
-    arg = str(arg)
-
-    return f'({arg})' if ARGS_SEPARATOR in arg else arg
-
 
 def join_args(args: list):
     return ARGS_SEPARATOR.join([
-        quote_arg(arg) for arg in args
+        str(arg) for arg in args
     ])
 
 
@@ -218,7 +153,7 @@ def rolling_window(
     period: int,
     fill=np.nan,
     # A stride for float is 8
-    stride: int=8
+    stride: int = 8
 ) -> np.ndarray:
     """Gets an `period`-period rolling window for 1d array
     """
@@ -230,3 +165,19 @@ def rolling_window(
         shape=(len(array) - period + 1, period),
         strides=(stride, stride)
     )
+
+
+DEFAULT_ARG_VALUE = ''
+
+
+def command_full_name(name, sub):
+    return f'{name}.{sub}' if sub else name
+
+
+NONE_TUPLE = (None, None)
+
+TYPE_DIRECTIVE = 1
+TYPE_COMMAND = 2
+TYPE_OPERATOR = 3
+TYPE_ARGUMENT = 4
+TYPE_SCALAR = 5
